% Test PSD estimation
clear all
%close all
load testsig    %sawtooth

N = length(x);
n = 1:N;

%Add Gaussian noise
sigma_n = 0.2;       %noise standard deviation here, use values in the range [0, .5] (0 ... no noise)
x = x + randn(1, N)*sigma_n;


%% Estimate PSD by Welch method
blocklen = 256;      %Takes blocks of this length (try "small" and "large blocks"!) 
averaging = 0;       %1=perform averaging; 0=no averaging 

if(blocklen > N)
  error('Block length must be smaller equal than signal length!')
end

%Select a window
win = hamming(blocklen);
%win = rectwin(blocklen);

if(averaging)
  [X, w] = pwelch(x, win);  
else
  [X, w] = pwelch(x(1:blocklen), win);
end

%Normalize power spectrum
X = X./max(X);

%% Plot input and PSD
figure
subplot(211), hold on, grid on
plot(n, x)
xlabel('n'), ylabel('x[n]'), title('Part of input signal')
xlim([1 200])

subplot(212), hold on, grid on
plot(w, 10*log10(X))
xlabel('\omega [rad/sample]')
ylabel('PSD estimate [dB/rad/sample]')
title(sprintf('Welch PSD estimate, block length = %d, sigma=%0.1f, average=%d', blocklen, sigma_n, averaging))
xlim([0 pi])

